package com.ruoyi.service.impl;

import java.util.*;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.domain.vo.PhotoStorageVO;
import com.ruoyi.framework.websocket.WebSocketUsers;
import com.ruoyi.service.UserPhotoStorageService;
import com.ruoyi.web.request.UserPhotoStorageCreateRequest;
import com.ruoyi.web.request.UserPhotoStorageListRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.mapper.UserPhotoStorageMapper;
import com.ruoyi.domain.UserPhotoStorage;
import org.springframework.transaction.annotation.Transactional;

/**
 * 用户照片存储Service业务层处理
 *
 * @author gxk
 * @date 2024-01-29
 */
@Service
@Transactional
public class UserPhotoStorageServiceImpl extends ServiceImpl<UserPhotoStorageMapper, UserPhotoStorage> implements UserPhotoStorageService
{
    @Autowired
    private UserPhotoStorageMapper userPhotoStorageMapper;

    /**
     * 查询用户照片存储列表
     *
     * @param request 用户照片存储
     * @return 用户照片存储
     */
    @Override
    public List<PhotoStorageVO> selectUserPhotoStorageList(UserPhotoStorageListRequest request)
    {
        List<UserPhotoStorage> userPhotoStorages = userPhotoStorageMapper.selectUserPhotoStorageList(request);
        // 把查询出的数组按照时间分组
        List<PhotoStorageVO> photoStorageVOList = new ArrayList<>();
        for (UserPhotoStorage photoStorage : userPhotoStorages) {
            String createTime = DateUtils.dateTime(photoStorage.getCreateTime());
            // photoStorageVOList为空时直接添加一个数据组
            if (CollUtil.isEmpty(photoStorageVOList)) {
                addPhotoStorageInNotCreateTime(photoStorageVOList, photoStorage, createTime);
            } else {
                // 查看此时间有没有对应的对象
                PhotoStorageVO storageVO = new PhotoStorageVO();
                int num = 0;
                for (PhotoStorageVO photoStorageVO : photoStorageVOList) {
                    if (photoStorageVO.getCreateTime().equals(createTime)) {
                        num++;
                        storageVO = photoStorageVO;
                    }
                }
                // num为0表示，没有此时间的对象，直接添加一个数据组；则取出此时间对象，往集合里新插入一条数据
                if (num == 0) {
                    addPhotoStorageInNotCreateTime(photoStorageVOList, photoStorage, createTime);
                } else {
                    List<UserPhotoStorage> userPhotoStorageList = storageVO.getUserPhotoStorageList();
                    userPhotoStorageList.add(photoStorage);
                }
            }
        }
        // 根据时间倒序排序
        photoStorageVOList.sort(Comparator.comparing(PhotoStorageVO::getCreateTime).reversed());
        return photoStorageVOList;
    }

    /**
     * 在photoStorageVOList数据没有此时间时新增一条新数据
     * @param photoStorageVOList
     * @param photoStorage
     * @param createTime
     */
    private void  addPhotoStorageInNotCreateTime(List<PhotoStorageVO> photoStorageVOList, UserPhotoStorage photoStorage, String createTime) {
        List<UserPhotoStorage> photoStorageChildren = new ArrayList<>();
        photoStorageChildren.add(photoStorage);
        PhotoStorageVO photoStorageVO = PhotoStorageVO.builder()
                .createTime(createTime)
                .userPhotoStorageList(photoStorageChildren).build();
        photoStorageVOList.add(photoStorageVO);
    }

    /**
     * 新增用户照片存储
     *
     * @param request 用户照片存储
     * @return 结果
     */
    @Override
    public int insertUserPhotoStorage(UserPhotoStorageCreateRequest request)
    {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        UserPhotoStorage userPhotoStorage = UserPhotoStorage.builder()
                .userId(loginUser.getUserId())
                .username(loginUser.getUsername())
                .path(request.getPath())
                .createTime(DateUtils.getNowDate())
                .build();
        WebSocketUsers.sendMessageToUserByUserId(loginUser.getUserId(), request.getPath());
        return userPhotoStorageMapper.insert(userPhotoStorage);
    }

    /**
     * 修改用户照片存储
     *
     * @param userPhotoStorage 用户照片存储
     * @return 结果
     */
    @Override
    public int updateUserPhotoStorage(UserPhotoStorage userPhotoStorage)
    {
        return userPhotoStorageMapper.updateUserPhotoStorage(userPhotoStorage);
    }

}
